"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var storage_1 = require("../../libs/storage");
exports.TYPE_REHYDRATE = 'ReduxModel/rehydrate';
var globalStore;
var defaultPersistOption = {
    __persist: { version: -1 },
};
var KEY_PREFIX = 'ReduxModel:Persist:';
var whiteList = {};
var model2persistDict = {};
var persistKeys = [];
var restoreDelay = 0;
var subscription = [];
var persistReducers = {};
var objectStrings = {};
var config = false;
var ready = false;
var readyEvents = [];
var timer;
var resetPersist = function () {
    persistReducers = {};
    subscription = [];
    restorePersist();
};
var persistIsReady = function () {
    ready = true;
    if (readyEvents.length) {
        readyEvents.forEach(function (item) { return item(); });
        readyEvents = [];
    }
};
var restorePersist = function () {
    if (!config) {
        return;
    }
    var strings = {};
    Object.keys(persistReducers).forEach(function (key) {
        strings[key] = objectStrings[key];
    });
    storage_1.setStorageItem(KEY_PREFIX + config.key, JSON.stringify(tslib_1.__assign(tslib_1.__assign({}, strings), defaultPersistOption)));
};
var parseStorageData = function (data) {
    if (!config) {
        return;
    }
    if (data === null) {
        resetPersist();
    }
    else {
        try {
            var tempReducers_1 = JSON.parse(data);
            if (tempReducers_1.__persist.version === config.version) {
                var shouldRestore_1 = false;
                delete tempReducers_1.__persist;
                persistReducers = {};
                Object.keys(tempReducers_1).forEach(function (key) {
                    if (persistKeys.indexOf(key) >= 0) {
                        objectStrings[key] = tempReducers_1[key];
                        persistReducers[key] = JSON.parse(tempReducers_1[key]);
                    }
                    else {
                        shouldRestore_1 = true;
                    }
                });
                if (shouldRestore_1) {
                    restorePersist();
                }
            }
            else {
                resetPersist();
            }
        }
        catch (e) {
            resetPersist();
            console.error('Unable to parser persist reducers from storage: ' + e.message);
        }
    }
    if (subscription.length) {
        var payload_1 = {};
        var canDispatch_1 = false;
        subscription.forEach(function (reducerName) {
            var persistKey = model2persistDict[reducerName];
            if (persistKey) {
                canDispatch_1 = true;
                payload_1[reducerName] = persistReducers[persistKey];
            }
        });
        subscription = [];
        if (canDispatch_1) {
            globalStore.dispatch({
                type: exports.TYPE_REHYDRATE,
                payload: payload_1,
            });
        }
    }
    persistIsReady();
};
exports.setPersistConfig = function (persist) {
    config = persist;
    if (config) {
        ready = false;
        storage_1.setStorage(config.storage);
        whiteList = config.whitelist || {};
        persistKeys = [];
        model2persistDict = Object.keys(whiteList).reduce(function (carry, persistKey) {
            persistKeys.push(persistKey);
            carry[whiteList[persistKey].getReducerName()] = persistKey;
            return carry;
        }, {});
        defaultPersistOption.__persist.version = config.version;
        restoreDelay = config.restoreDelay === undefined ? 0 : Number(config.restoreDelay) || 0;
    }
    else {
        persistIsReady();
    }
};
exports.handlePersist = function (store) {
    globalStore = store;
    if (config && !ready) {
        var storageData = storage_1.getStorageItem(KEY_PREFIX + config.key);
        if (storageData === null || typeof storageData === 'string') {
            parseStorageData(storageData);
        }
        else {
            storageData.then(parseStorageData);
        }
    }
};
// Since whitelist is required, model instances who want to link persist are always created before store.
exports.switchInitData = function (reducerName, state) {
    if (!config) {
        return state;
    }
    var persistKey = model2persistDict[reducerName];
    if (!persistKey) {
        return state;
    }
    if (ready) {
        var persistState = persistReducers[persistKey];
        return persistState === undefined ? state : persistState;
    }
    if (subscription.indexOf(reducerName) === -1) {
        subscription.push(reducerName);
    }
    return state;
};
exports.updatePersistState = function (state, force) {
    if (!config) {
        return;
    }
    // Persist is not ready before TYPE_REHYDRATE
    if (!ready && !force) {
        return;
    }
    // Sync models + async models
    var tempPersistReducers = tslib_1.__assign({}, persistReducers);
    var changed = false;
    Object.keys(model2persistDict).forEach(function (reducerName) {
        var persistKey = model2persistDict[reducerName];
        tempPersistReducers[persistKey] = state[reducerName];
        if (state[reducerName] !== persistReducers[persistKey]) {
            var tempString = JSON.stringify(state[reducerName]);
            if (!changed && objectStrings[persistKey] !== tempString) {
                changed = true;
            }
            objectStrings[persistKey] = tempString;
        }
    });
    persistReducers = tempPersistReducers;
    if (!changed) {
        return;
    }
    if (restoreDelay <= 0) {
        return restorePersist();
    }
    if (timer === undefined) {
        timer = setTimeout(function () {
            restorePersist();
            timer = undefined;
        }, restoreDelay);
    }
};
exports.onPersistReady = function (fn) {
    if (ready) {
        fn();
    }
    else {
        readyEvents.push(fn);
    }
};
exports.isPersistReady = function () {
    return ready;
};
exports.persistContainReducer = function (reducerName) {
    return model2persistDict[reducerName] !== undefined;
};
