"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var object_assign_1 = tslib_1.__importDefault(require("object-assign"));
var redux_1 = require("redux");
var StoreNotFoundError_1 = require("../exceptions/StoreNotFoundError");
var dev_1 = require("../../libs/dev");
var persist_1 = require("./persist");
var hasEffectsReducers = [];
var autoReducers = {};
var usersReducers = {};
var store;
var listeners = [];
var onCombineReducers;
var stateWhenDispatching;
var isDispatching = false;
var combine = function () {
    if (dev_1.isDebug()) {
        Object.keys(usersReducers).forEach(function (key) {
            if (autoReducers[key] && hasEffectsReducers.indexOf(key) === -1) {
                // Indeed, it's reducer name but not model name
                console.warn("Model '" + key + "' has been registered automatically, do not register it again.");
            }
        });
    }
    var combined = redux_1.combineReducers(tslib_1.__assign(tslib_1.__assign({}, autoReducers), usersReducers));
    if (onCombineReducers) {
        combined = onCombineReducers(combined);
    }
    return function (state, action) {
        isDispatching = true;
        stateWhenDispatching = state;
        var result = combined(state, action);
        if (stateWhenDispatching !== result) {
            persist_1.updatePersistState(result, action.type === persist_1.TYPE_REHYDRATE);
        }
        isDispatching = false;
        return result;
    };
};
exports.watchEffectsReducer = function (reducerName, className) {
    hasEffectsReducers.push(reducerName);
    if (usersReducers[reducerName]) {
        return;
    }
    exports.onStoreCreated(function () {
        if (persist_1.persistContainReducer(reducerName)) {
            return;
        }
        setTimeout(function () {
            if (!usersReducers[reducerName]) {
                console.error("Model '" + className + "' has override protected method 'effects()', consider register its instance manually:\n\n\nExample:\n\nconst store = createReduxStore({\n  reducers: {\n    ...aModel.register(),\n    ...bModel.register(),\n    ...cModel.register(),\n  }\n});\n\n\n");
            }
        });
    });
};
exports.appendReducers = function (reducers) {
    var _a;
    object_assign_1.default(autoReducers, reducers);
    (_a = store) === null || _a === void 0 ? void 0 : _a.replaceReducer(combine());
};
function createReduxStore(config) {
    usersReducers = config.reducers || {};
    onCombineReducers = config.onCombineReducers;
    persist_1.setPersistConfig(config.persist);
    if (store) {
        // Avoid to dispatch persist data for @@redux/x.y.z
        persist_1.handlePersist(store);
        store.replaceReducer(combine());
    }
    else {
        store = redux_1.createStore(combine(), config.preloadedState, config.enhancer);
        listeners.forEach(function (listener) { return listener(store); });
        listeners = [];
        persist_1.handlePersist(store);
    }
    return store;
}
exports.createReduxStore = createReduxStore;
exports.getStore = function () {
    if (!store) {
        throw new StoreNotFoundError_1.StoreNotFoundError();
    }
    return store;
};
exports.getState = function () {
    return isDispatching ? stateWhenDispatching : exports.getStore().getState();
};
exports.onStoreCreated = function (fn) {
    if (store) {
        fn(store);
    }
    else {
        listeners.push(fn);
    }
};
